#!/usr/bin/python
import urllib
import sys
import popen2
import re
import htmlentitydefs

def convert_entity(m):
    text = m.group(0)
    try:
        return htmlentitydefs.entitydefs[text[1:-1]]
    except KeyError:
        pass
    return text

def load_and_apply_XSLT(url, xsltfile):
    page = urllib.urlopen(url).read()
    (childstdout, childstdin) = popen2.popen2('xsltproc --nonet --html %s - 2> /dev/null | xmllint --format -' % xsltfile)
    childstdin.write(page)
    childstdin.close()
    xml = childstdout.read()
    childstdout.close()
    return xml

def check_links(xml, url, pagename, requireurls=True):
    links = re.findall(r'<link[^>]*>', xml)
    nlinks = len(links)
    if nlinks < 1:
        print '\nError: No links on %s page %s!' % (pagename, url)
        sys.exit(1)

    ntitles = len(re.findall(r'<title>[^<]+</title>', xml))
    if ntitles != nlinks:
        print '\nError: Some links do not have titles on %s page %s.' % (pagename, url)
        sys.exit(1)

    urls = re.findall(r'<url>[^<]+</url>', xml)
    nurls = len(urls)
    if requireurls and (nurls != nlinks):
        print  '\nError: Some links do not have urls on %s page %s.' % (pagename, url)
        sys.exit(1)

    return (links, urls)

print 'Running unit test of Google Video service defintions.\n'

######

print 'Checking main page...',

mainurl = 'http://video.google.com/'
mainpagexml = load_and_apply_XSLT(mainurl, '../services/google/categories.xsl')
# The search URL is supposed to be empty. Therefore skip the URL check.
(links, urls) = check_links(mainpagexml, mainurl, 'main', requireurls=False)

print 'OK'

######

# select the third URL ("Most shared") for testing the navigation template
naviurl = re.sub("&\w+;", convert_entity, urls[1][5:-6])

print "Checking navigation page %s..." % naviurl, 

navipagexml = load_and_apply_XSLT(naviurl, '../services/google/navigation.xsl')
(links, urls) = check_links(navipagexml, naviurl, 'navigation')

print 'OK'

######

print "Checking video pages...",

googlelinks = 0
youtubelinks = 0
for i in xrange(len(links)):
    if 'type="video"' in links[i]:
        m = re.search('template="(\w+)"', links[i])
        template = m.group(1)
        if template == 'googlevideo':
            googlelinks += 1
        elif template == 'youtubevideo':
            youtubelinks += 1
        else:
            print '\nError: Unknown template (%s) on page %s.' % (template, naviurl)

        videourl = re.sub("&\w+;", convert_entity, urls[i][5:-6])
        videoxml = load_and_apply_XSLT(videourl, '../services/google/%s.xsl' % template)

        if re.search(r'<title>[^<]+</title>', videoxml) is None:
            print '\nError: No title on video page %s.' % videourl
            sys.exit(1)
        if re.search(r'<url>[^<]+</url>', videoxml) is None:
            print '\nError: No URL on video page %s.' % videourl
            sys.exit(1)

print 'OK'
if googlelinks == 0:
    print 'Warning: No Google urls tested.'
if youtubelinks == 0:
    print 'Warning: No Youtube urls tested.'

######

print "Checking search results page...",

searchurl = 'http://video.google.com/videosearch?q=google+video&dur=1&so=3'
searchpagexml = load_and_apply_XSLT(searchurl, '../services/google/searchresults.xsl')
(links, urls) = check_links(searchpagexml, searchurl, 'search results')

print 'OK'
print
